/**
 ******************************************************************************
 *
 * @file        MG82F6P32_UART1_DRV.h
 *
 * @brief       This is the C code format driver file.
 *
 * @par         Project
 *              MG82F6P32
 * @version     V1.00
 * @date        2024/04/01
 * @author      Megawin Software Center
 * @copyright   Copyright (c) 2024 MegaWin Technology Co., Ltd.
 *              All rights reserved.
 *
 ******************************************************************************
 * @par         Disclaimer
 *      The Demo software is provided "AS IS"  without any warranty, either
 *      expressed or implied, including, but not limited to, the implied warranties
 *      of merchantability and fitness for a particular purpose.  The author will
 *      not be liable for any special, incidental, consequential or indirect
 *      damages due to loss of data or any other reason.
 *      These statements agree with the world wide and local dictated laws about
 *      authorship and violence against these laws.
 ******************************************************************************
 ******************************************************************************
 */

#ifndef MG82F6P32_UART1_DRV_H
#define MG82F6P32_UART1_DRV_H

/**
*******************************************************************************
* @brief        UART1 Enable Interrupt
* @details      UART1 Enable Interrupt
* @param[in]    __STATE__ :
*  @arg\b       MW_DISABLE
*  @arg\b       MW_ENABLE
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_Interrupt_Cmd(MW_DISABLE)
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_IT_Cmd(__STATE__)\
    MWT(\
        (__STATE__==0?(EIE1=EIE1&~(ES1)):(EIE1=EIE1|(ES1)));\
    ;)

/**
*******************************************************************************
* @brief        UART1 Mode Select
* @details      Set S1TME SM31 SM01 SM11
* @param[in]    __SELECT__ :
*  @arg\b       URT1_MODE0_SHIFT_REG
*  @arg\b       URT1_MODE1_8BIT
*  @arg\b       URT1_MODE2_9BIT
*  @arg\b       URT1_MODE3_9BIT
*  @arg\b       URT1_MODE4_SPI_MASTER
*  @arg\b       URT1_MODE8_8BIT_PURE_TIMER
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_Mode_Select(URT1_Mode0_SHIFT_REG)
* @endcode
*******************************************************************************
*/

///@cond __DRV_URT1_Mode_Select
#define UART1_MODE0_SHIFT_REG 0x00
#define UART1_MODE1_8BIT 0x01
#define UART1_MODE2_9BIT 0x02
#define UART1_MODE3_9BIT 0x03
#define UART1_MODE4_SPI_MASTER 0x04
#define URT1_MODE8_8BIT_PURE_TIMER 0x08
///@endcond

#define __DRV_URT1_Mode_Select(__SELECT__)\
    MWT(\
        __DRV_URT1_SetS1TME(__SELECT__);\
        __DRV_URT1_SetSM31(__SELECT__);\
        __DRV_URT1_SetSM01(__SELECT__);\
        __DRV_URT1_SetSM11(__SELECT__);\
    ;)

/**
*******************************************************************************
* @brief        UART1 PinMUX Select
* @details      Set S1PS1 S1PS0
* @param[in]    __SELECT__ :
*  @arg\b       UART1_RX_P12_TX_P13
*  @arg\b       UART1_RX_P60_TX_P61
*  @arg\b       UART1_RX_P10_TX_P11
*  @arg\b       UART1_RX_P34_TX_P35
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_PinMux_Select(UART1_RX_P10_TX_P11)
* @endcode
*******************************************************************************
*/

///@cond __DRV_URT1_PinMux_Select
#define UART1_RX_P12_TX_P13 0x00
#define UART1_RX_P60_TX_P61 0x01
#define UART1_RX_P10_TX_P11 0x02
#define UART1_RX_P34_TX_P35 0x03
///@endcond

#define __DRV_URT1_PinMux_Select(__SELECT__)\
    MWT(\
        __DRV_URT1_SetS1PS1(__SELECT__);\
        __DRV_URT1_SetS1PS0(__SELECT__);\
    ;)

/**
*******************************************************************************
* @brief        UART1 Mode Select
* @details      Set S1TME
* @param[in]    __STATE__ :
*  @arg\b       URT1_MODE0_SHIFT_REG
*  @arg\b       URT1_MODE1_8BIT
*  @arg\b       URT1_MODE2_9BIT
*  @arg\b       URT1_MODE3_9BIT
*  @arg\b       URT1_MODE4_SPI_MASTER
*  @arg\b       URT1_MODE8_8BIT_PURE_TIMER
* @return       None
* @note         S1TME=1 UART1 timer mode
* @par          Example
* @code
                __DRV_URT1_SetS1TME(URT1_MODE1_8BIT)
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_SetS1TME(__STATE__)\
    MWT(\
        __DRV_SFR_PageIndex(S1CFG_Page);\
        ((__STATE__^0x00)==0?(S1CFG=S1CFG&~(S1TME)):\
        (__STATE__^0x01)==0?(S1CFG=S1CFG&~(S1TME)):\
        (__STATE__^0x02)==0?(S1CFG=S1CFG&~(S1TME)):\
        (__STATE__^0x03)==0?(S1CFG=S1CFG&~(S1TME)):\
        (__STATE__^0x04)==0?(S1CFG=S1CFG&~(S1TME)):\
        (__STATE__^0x08)==0?(S1CFG=S1CFG|(S1TME)):_nop_());\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 Mode Select
* @details      Set SM31
* @param[in]    __STATE__ :
*  @arg\b       URT1_MODE0_SHIFT_REG
*  @arg\b       URT1_MODE1_8BIT
*  @arg\b       URT1_MODE2_9BIT
*  @arg\b       URT1_MODE3_9BIT
*  @arg\b       URT1_MODE4_SPI_MASTER
*  @arg\b       URT1_MODE8_8BIT_PURE_TIMER
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_SetSM31(URT1_MODE1_8BIT)
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_SetSM31(__STATE__)\
    MWT(\
        __DRV_SFR_PageIndex(S1CFG_Page);\
        ((__STATE__^0x00)==0?(S1CFG=S1CFG&~(SM31)):\
        (__STATE__^0x01)==0?(S1CFG=S1CFG&~(SM31)):\
        (__STATE__^0x02)==0?(S1CFG=S1CFG&~(SM31)):\
        (__STATE__^0x03)==0?(S1CFG=S1CFG&~(SM31)):\
        (__STATE__^0x04)==0?(S1CFG=S1CFG|(SM31)):\
        (__STATE__^0x08)==0?(S1CFG=S1CFG&~(SM31)):_nop_());\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 Mode Select
* @details      Set SM01
* @param[in]    __STATE__ :
*  @arg\b       URT1_MODE0_SHIFT_REG
*  @arg\b       URT1_MODE1_8BIT
*  @arg\b       URT1_MODE2_9BIT
*  @arg\b       URT1_MODE3_9BIT
*  @arg\b       URT1_MODE4_SPI_MASTER
*  @arg\b       URT1_MODE8_8BIT_PURE_TIMER
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_SetSM01(URT1_MODE2_9BIT)
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_SetSM01(__STATE__)\
    MWT(\
        __DRV_SFR_PageIndex(S1CON_Page);\
        ((__STATE__^0x00)==0?(SM01=0):\
        (__STATE__^0x01)==0?(SM01=0):\
        (__STATE__^0x02)==0?(SM01=1):\
        (__STATE__^0x03)==0?(SM01=1):\
        (__STATE__^0x04)==0?(SM01=0):\
        (__STATE__^0x08)==0?(SM01=0):_nop_());\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 Mode Select
* @details      Set SM11
* @param[in]    __STATE__ :
*  @arg\b       URT1_MODE0_SHIFT_REG
*  @arg\b       URT1_MODE1_8BIT
*  @arg\b       URT1_MODE2_9BIT
*  @arg\b       URT1_MODE3_9BIT
*  @arg\b       URT1_MODE4_SPI_MASTER
*  @arg\b       URT1_MODE8_8BIT_PURE_TIMER
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_SetSM11(URT1_MODE3_9BIT)
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_SetSM11(__STATE__)\
    MWT(\
        __DRV_SFR_PageIndex(S1CON_Page);\
        ((__STATE__^0x00)==0?(SM11=0):\
        (__STATE__^0x01)==0?(SM11=1):\
        (__STATE__^0x02)==0?(SM11=0):\
        (__STATE__^0x03)==0?(SM11=1):\
        (__STATE__^0x04)==0?(SM11=0):\
        (__STATE__^0x08)==0?(SM11=0):_nop_());\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 PinMUX Select
* @details      Set S1PS1
* @param[in]    __STATE__ :
*  @arg\b       UART1_RX_P12_TX_P13
*  @arg\b       UART1_RX_P60_TX_P61
*  @arg\b       UART1_RX_P10_TX_P11
*  @arg\b       UART1_RX_P34_TX_P35
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_SetS1PS1(UART1_RX_P12_TX_P13)
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_SetS1PS1(__STATE__)\
    MWT(\
        __DRV_SFR_PageIndex(AUXR9_Page);\
        ((__STATE__^0x00)==0?AUXR9=AUXR9&~(S1PS1):\
        (__STATE__^0x01)==0?AUXR9=AUXR9&~(S1PS1):\
        (__STATE__^0x02)==0?AUXR9=AUXR9|(S1PS1):\
        (__STATE__^0x03)==0?AUXR9=AUXR9|(S1PS1):_nop_());\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 PinMUX Select
* @details      Set S1PS0
* @param[in]    __STATE__ :
*  @arg\b       UART1_RX_P12_TX_P13
*  @arg\b       UART1_RX_P60_TX_P61
*  @arg\b       UART1_RX_P10_TX_P11
*  @arg\b       UART1_RX_P34_TX_P35
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_SetS1PS0(UART1_RX_P10_TX_P11)
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_SetS1PS0(__STATE__)\
    MWT(\
        __DRV_SFR_PageIndex(AUXR9_Page);\
        ((__STATE__^0x00)==0?AUXR9=AUXR9&~(S1PS0):\
        (__STATE__^0x01)==0?AUXR9=AUXR9|(S1PS0):\
        (__STATE__^0x02)==0?AUXR9=AUXR9&~(S1PS0):\
        (__STATE__^0x03)==0?AUXR9=AUXR9|(S1PS0):_nop_());\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 Data Order Select
* @details      Set S1DOR
* @param[in]    __SELECT__ :
*  @arg\b       UART1_DATA_ORDER_LSB
*  @arg\b       UART1_DATA_ORDER_MSB
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_DataOrder_Select(UART1_DATA_ORDER_LSB)
* @endcode
*******************************************************************************
*/

///@cond __DRV_URT1_DataOrder_Select
#define UART1_DATA_ORDER_LSB 1
#define UART1_DATA_ORDER_MSB 0
///@endcond

#define __DRV_URT1_DataOrder_Select(__SELECT__)\
    MWT(\
        __DRV_SFR_PageIndex(S1CFG_Page);\
        (__SELECT__==1)?(S1CFG=S1CFG|(S1DOR)):(S1CFG=S1CFG&~(S1DOR));\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 Pure Timer Select
* @details      Set S1DOR
* @param[in]    __SELECT__ :
*  @arg\b       UART1_8BIT_PURE_TIMER
* @return       None
* @note         S1DOR=1 S1TME=1 8bit pure timer
* @par          Example
* @code
                __DRV_URT1_PureTimer_Select(UART1_8BIT_PURE_TIMER)
* @endcode
*******************************************************************************
*/

///@cond __DRV_URT1_PureTimer_Select
#define UART1_8BIT_PURE_TIMER 0x08
///@endcond

#define __DRV_URT1_PureTimer_Select(__SELECT__)\
    MWT(\
        __DRV_SFR_PageIndex(S1CFG_Page);\
        (__SELECT__==0x08)?(S1CFG=S1CFG|(S1DOR)):(S1CFG=S1CFG|(S1DOR));\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 Enhance Baud Rate Select
* @details      Set S1MOD1
* @param[in]    __SELECT__ :
*  @arg\b       UART1_DEFAULT_BAUD_RATE
*  @arg\b       UART1_DOUBLE_BAUD_RATE_X1
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_BaudRate_Select(Default_Baud_Rate)
* @endcode
*******************************************************************************
*/

///@cond __DRV_URT1_BaudRate_Select
#define UART1_DEFAULT_BAUD_RATE 0
#define UART1_DOUBLE_BAUD_RATE_X1 1
///@endcond

#define __DRV_URT1_BaudRate_Select(__SELECT__)\
    MWT(\
        __DRV_SFR_PageIndex(S1CFG_Page);\
        ((__SELECT__^1)==0?(S1CFG=S1CFG|(S1MOD1)):(S1CFG=S1CFG&~(S1MOD1)));\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 S1BRG Clock Source Select
* @details      Set S1TX12
* @param[in]    __SELECT__ :
*  @arg\b       S1BRG_CLOCK_SOURCE_SYSCLK_DIV_1
*  @arg\b       S1BRG_CLOCK_SOURCE_SYSCLK_DIV_12
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_BaudRateGeneratorClock_Select(S1BRG_CLOCK_SOURCE_SYSCLK_DIV_1)
* @endcode
*******************************************************************************
*/

///@cond __DRV_URT1_BaudRateGeneratorClock_Select
#define S1BRG_CLOCK_SOURCE_SYSCLK_DIV_1 1
#define S1BRG_CLOCK_SOURCE_SYSCLK_DIV_12 0
///@endcond

#define __DRV_URT1_BaudRateGeneratorClock_Select(__SELECT__)\
    MWT(\
        __DRV_SFR_PageIndex(S1CFG_Page);\
        (__SELECT__)==0?(S1CFG=S1CFG&~(S1TX12)):(S1CFG=S1CFG|(S1TX12));\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 Pure Timer Clock Source Select
* @details      Set S1TX12 SM21
* @param[in]    __SELECT__ :
*  @arg\b       PURE_TIMER_CLOCK_SOURCE_SYSCLK_DIV_12
*  @arg\b       PURE_TIMER_CLOCK_SOURCE_T1OF
*  @arg\b       PURE_TIMER_CLOCK_SOURCE_SYSCLK_DIV_1
*  @arg\b       PURE_TIMER_CLOCK_SOURCE_T0OF
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_Pure_Timer_Clock_Source_Select(PURE_TIMER_CLOCK_SOURCE_SYSCLK_DIV_12)
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_Pure_Timer_Clock_Source_Select(__SELECT__)\
    MWT(\
        __DRV_URT1_SetS1TX12(__SELECT__);\
        __DRV_URT1_SetSM21(__SELECT__);\
    ;)

/**
*******************************************************************************
* @brief        UART1 Pure Timer Clock Source
* @details      Set S1TX12
* @param[in]    __STATE__ :
*  @arg\b       PURE_TIMER_CLOCK_SOURCE_SYSCLK_DIV_12
*  @arg\b       PURE_TIMER_CLOCK_SOURCE_T1OF
*  @arg\b       PURE_TIMER_CLOCK_SOURCE_SYSCLK_DIV_1
*  @arg\b       PURE_TIMER_CLOCK_SOURCE_T0OF
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_SetS1TX12(PURE_TIMER_CLOCK_SOURCE_SYSCLK_DIV_12)
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_SetS1TX12(__STATE__)\
    MWT(\
        __DRV_SFR_PageIndex(S1CFG_Page);\
        ((__STATE__^0)==0?S1CFG=S1CFG&~(S1TX12):\
        (__STATE__^1)==0?S1CFG=S1CFG&~(S1TX12):\
        (__STATE__^2)==0?S1CFG=S1CFG|(S1TX12):\
        (__STATE__^3)==0?S1CFG=S1CFG|(S1TX12):_nop_());\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 Pure Timer Clock Source
* @details      Set SM21
* @param[in]    __STATE__ :
*  @arg\b       PURE_TIMER_CLOCK_SOURCE_SYSCLK_DIV_12
*  @arg\b       PURE_TIMER_CLOCK_SOURCE_T1OF
*  @arg\b       PURE_TIMER_CLOCK_SOURCE_SYSCLK_DIV_1
*  @arg\b       PURE_TIMER_CLOCK_SOURCE_T0OF
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_SetSM21(PURE_TIMER_CLOCK_SOURCE_SYSCLK_DIV_12)
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_SetSM21(__STATE__)\
    MWT(\
        __DRV_SFR_PageIndex(S1CON_Page);\
        ((__STATE__^0)==0?(SM21=0):\
        (__STATE__^1)==0?(SM21=1):\
        (__STATE__^2)==0?(SM21=0):\
        (__STATE__^3)==0?(SM21=1):_nop_());\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 Enable S1BRG clock output control
* @details      Set S1CKOE
* @param[in]    __STATE__ :
*  @arg\b       MW_DISABLE
*  @arg\b       MW_ENABLE
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_Baud_Rate_Generator_clock_output_Cmd(MW_ENABLE)
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_Baud_Rate_Generator_Clock_Output_Cmd(__STATE__)\
    MWT(\
        __DRV_SFR_PageIndex(S1CFG_Page);\
        (__STATE__==1)?(S1CFG=S1CFG|(S1CKOE)):(S1CFG=S1CFG&~(S1CKOE));\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 Enable S1BRG Operation
* @details      Set S1TR
* @param[in]    __STATE__ :
*  @arg\b       MW_DISABLE
*  @arg\b       MW_ENABLE
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_BaudRateGenerator_Cmd(MW_DISABLE)
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_BaudRateGenerator_Cmd(__STATE__)\
    MWT(\
        __DRV_SFR_PageIndex(S1CFG_Page);\
        (__STATE__==1)?(S1CFG=S1CFG|(S1TR)):(S1CFG=S1CFG&~(S1TR));\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 Baud Rate Generator Reload Register
* @details      Write S1BRT S1BRC
* @param[in]    __RELOAD__ :
*  @arg\b       S1BRG_BRGRL_9600_2X_12000000_1T
*  @arg\b       S1BRG_BRGRL_19200_2X_12000000_1T
*  @arg\b       S1BRG_BRGRL_57600_2X_12000000_1T
*  @arg\b       S1BRG_BRGRL_9600_2X_11059200_1T
*  @arg\b       S1BRG_BRGRL_19200_2X_11059200_1T
*  @arg\b       S1BRG_BRGRL_57600_2X_11059200_1T
*  @arg\b       S1BRG_BRGRL_115200_2X_11059200_1T
*  @arg\b       S1BRG_BRGRL_9600_2X_24000000_1T
*  @arg\b       S1BRG_BRGRL_19200_2X_24000000_1T
*  @arg\b       S1BRG_BRGRL_57600_2X_24000000_1T
*  @arg\b       S1BRG_BRGRL_115200_2X_24000000_1T
*  @arg\b       S1BRG_BRGRL_9600_2X_22118400_1T
*  @arg\b       S1BRG_BRGRL_19200_2X_22118400_1T
*  @arg\b       S1BRG_BRGRL_57600_2X_22118400_1T
*  @arg\b       S1BRG_BRGRL_115200_2X_22118400_1T
*  @arg\b       S1BRG_BRGRL_9600_2X_29491200_1T
*  @arg\b       S1BRG_BRGRL_19200_2X_29491200_1T
*  @arg\b       S1BRG_BRGRL_57600_2X_29491200_1T
*  @arg\b       S1BRG_BRGRL_115200_2X_29491200_1T
*  @arg\b       S1BRG_BRGRL_9600_2X_33177600_1T
*  @arg\b       S1BRG_BRGRL_19200_2X_33177600_1T
*  @arg\b       S1BRG_BRGRL_57600_2X_33177600_1T
*  @arg\b       S1BRG_BRGRL_115200_2X_33177600_1T
*  @arg\b       S1BRG_BRGRL_9600_2X_32000000_1T
*  @arg\b       S1BRG_BRGRL_19200_2X_32000000_1T
*  @arg\b       S1BRG_BRGRL_57600_2X_32000000_1T
*  @arg\b       S1BRG_BRGRL_115200_2X_32000000_1T
*  @arg\b       S1BRG_BRGRL_9600_2X_36000000_1T
*  @arg\b       S1BRG_BRGRL_19200_2X_36000000_1T
*  @arg\b       S1BRG_BRGRL_57600_2X_36000000_1T
*  @arg\b       S1BRG_BRGRL_115200_2X_36000000_1T
*  @arg\b       S1BRG_BRGRL_19200_2X_44236800_1T
*  @arg\b       S1BRG_BRGRL_57600_2X_44236800_1T
*  @arg\b       S1BRG_BRGRL_115200_2X_44236800_1T
*  @arg\b       S1BRG_BRGRL_19200_2X_48000000_1T
*  @arg\b       S1BRG_BRGRL_57600_2X_48000000_1T
*  @arg\b       S1BRG_BRGRL_115200_2X_48000000_1T
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_BaudRateReloadReg_Write(S1BRG_BRGRL_9600_2X_12000000_1T)
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_BaudRateReloadReg_Write(__RELOAD__)\
    MWT(\
        __DRV_SFR_PageIndex(S1BRT_Page);\
        S1BRT=S1BRC=__RELOAD__;\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 Enable Reception
* @details      Set REN1
* @param[in]    __STATE__ :
*  @arg\b       MW_DISABLE
*  @arg\b       MW_ENABLE
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_SerialReception_Cmd(MW_ENABLE)
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_SerialReception_Cmd(__STATE__)\
    MWT(\
        __DRV_SFR_PageIndex(S1CON_Page);\
        (__STATE__==1)?(REN1=1):(REN1=0);\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 In Mode 0 Mode 4 Enable SYSCLK/4 SYSCLK/12
* @details      Set S1MOD1
* @param[in]    __STATE__ :
*  @arg\b       MW_DISABLE
*  @arg\b       MW_ENABLE
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_BaudRateSysclkDiv4_Cmd(MW_DISABLE)
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_BaudRateSysclkDiv4_Cmd(__STATE__)\
    MWT(\
        __DRV_SFR_PageIndex(S1CFG_Page);\
        __STATE__==1?(S1CFG=S1CFG|(S1MOD1)):(S1CFG=S1CFG&~(S1MOD1));\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 Mode04 SPI Master Data Mode Select
* @details      Set SM21:TB81 (CPOL:CPHA)
* @param[in]    __SELECT__ :
*  @arg\b       DATA_LEADING_EDGE_SAMPLE_RISING_TRAILING_EDGE_SETUP_FALLING
*  @arg\b       DATA_LEADING_EDGE_SETUP_RISING_TRAILING_EDGE_SAMPLE_FALLING
*  @arg\b       DATA_LEADING_EDGE_SAMPLE_FALLING_TRAILING_EDGE_SETUP_RISING
*  @arg\b       DATA_LEADING_EDGE_SETUP_FALLING_TRAILING_EDGE_SAMPLE_RISING
* @return       None
* @note         None
* @par         Example
* @code
               __DRV_URT1_Mode04_Data_Mode_Select(DATA_LEADING_EDGE_SAMPLE_RISING_TRAILING_EDGE_SETUP_FALLING)
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_Mode04_Data_Mode_Select(__SELECT__)\
    MWT(\
        __DRV_URT1_Mode04_SetSM21(__SELECT__);\
        __DRV_URT1_Mode04_SetTB81(__SELECT__);\
    ;)

/**
*****************************************************************************
* @brief        UART1 Mode04 SPI Master Data Mode Select
* @details      Set SM21
* @param[in]    __STATE__ :
*  @arg\b       DATA_LEADING_EDGE_SAMPLE_RISING_TRAILING_EDGE_SETUP_FALLING
*  @arg\b       DATA_LEADING_EDGE_SETUP_RISING_TRAILING_EDGE_SAMPLE_FALLING
*  @arg\b       DATA_LEADING_EDGE_SAMPLE_FALLING_TRAILING_EDGE_SETUP_RISING
*  @arg\b       DATA_LEADING_EDGE_SETUP_FALLING_TRAILING_EDGE_SAMPLE_RISING
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_Mode04_SetSM21(DATA_LEADING_EDGE_SETUP_FALLING_TRAILING_EDGE_SAMPLE_RISING)
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_Mode04_SetSM21(__STATE__)\
    MWT(\
        __DRV_SFR_PageIndex(S1CON_Page);\
        ((__STATE__^0x00)==0?(SM21=1):\
        (__STATE__^0x01)==0?(SM21=1):\
        (__STATE__^0x02)==0?(SM21=0):\
        (__STATE__^0x03)==0?(SM21=0):_nop_());\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*****************************************************************************
* @brief        UART1 Mode04 SPI Master Data Mode Select
* @details      Set TB81
* @param[in]    __STATE__ :
*  @arg\b       DATA_LEADING_EDGE_SAMPLE_RISING_TRAILING_EDGE_SETUP_FALLING
*  @arg\b       DATA_LEADING_EDGE_SETUP_RISING_TRAILING_EDGE_SAMPLE_FALLING
*  @arg\b       DATA_LEADING_EDGE_SAMPLE_FALLING_TRAILING_EDGE_SETUP_RISING
*  @arg\b       DATA_LEADING_EDGE_SETUP_FALLING_TRAILING_EDGE_SAMPLE_RISING
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_Mode04_SetTB81(DATA_LEADING_EDGE_SAMPLE_RISING_TRAILING_EDGE_SETUP_FALLING)
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_Mode04_SetTB81(__STATE__)\
    MWT(\
        __DRV_SFR_PageIndex(S1CON_Page);\
        ((__STATE__^0x00)==0?(TB81=1):\
        (__STATE__^0x01)==0?(TB81=0):\
        (__STATE__^0x02)==0?(TB81=1):\
        (__STATE__^0x03)==0?(TB81=0):_nop_());\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 In Timer Mode Enable Pure Timer Baudrate Generator Overflow to S1 Interrupt
* @details      Set TB81
* @param[in]    __STATE__ :
*  @arg\b       MW_DISABLE
*  @arg\b       MW_ENABLE
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_Pure_Timer_Baudrate_Generator_Overflow_to_S1_Interrupt_Cmd(MW_DISABLE)
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_Pure_Timer_Baudrate_Generator_Overflow_to_S1_Interrupt_Cmd(__STATE__)\
    MWT(\
        __DRV_SFR_PageIndex(S1CON_Page);\
        ((__STATE__)==1?(TB81=1):(TB81=0));\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 In Timer Mode Enable Pure Timer External Pin Trigger to S1 Interrupt
* @details      Set REN1
* @param[in]    __STATE__ :
*  @arg\b       MW_DISABLE
*  @arg\b       MW_ENABLE
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_Pure_Timer_External_Pin_Trigger_to_S1_Interrupt_Cmd(MW_DISABLE)
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_Pure_Timer_External_Pin_Trigger_to_S1_Interrupt_Cmd(__STATE__)\
    MWT(\
        __DRV_SFR_PageIndex(S1CON_Page);\
        ((__STATE__)==1?(REN1=1):(REN1=0));\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 In Timer Mode Enable Pure Timer External Pin Control Baudrate Generator
* @details      Set S1MOD1
* @param[in]    __STATE__ :
*  @arg\b       MW_DISABLE
*  @arg\b       MW_ENABLE
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_Pure_Timer_External_Pin_Control_Baudrate_Generator_Cmd(MW_DISABLE)
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_Pure_Timer_External_Pin_Control_Baudrate_Generator_Cmd(__STATE__)\
    MWT(\
        __DRV_SFR_PageIndex(S1CFG_Page);\
        ((__STATE__)==1?(S1CFG=S1CFG|(S1MOD1)):(S1CFG=S1CFG&~(S1MOD1)));\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 In Timer Mode Enable Pure Timer External Pin Trigger Level
* @details      Set RB81
* @param[in]    __STATE__ :
*  @arg\b       MW_DISABLE
*  @arg\b       MW_ENABLE
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_Pure_Timer_External_Pin_Trigger_Level_Cmd(MW_DISABLE)
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_Pure_Timer_External_Pin_Trigger_Level_Cmd(__STATE__)\
    MWT(\
        __DRV_SFR_PageIndex(S1CON_Page);\
        ((__STATE__)==1?(RB81=1):(RB81=0));\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 read interrupt flag
* @details      Read TI1
* @param[in]    __STATE__ :
* @arg\b        Read flag
* @return       None
* @note         Data type bit
* @par          Example
* @code
                __DRV_URT1_GetTI1(usertmp);
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_GetTI1(__STATE__)\
    MWT(\
        __DRV_SFR_PageIndex(S1CON_Page);\
        __STATE__=TI1;\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 read interrupt flag
* @details      Read RI1
* @param[in]    __STATE__ :
* @arg\b        Read flag
* @return       None
* @note         Data type bit
* @par          Example
* @code
                __DRV_URT1_GetRI1(usertmp);
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_GetRI1(__STATE__)\
    MWT(\
        __DRV_SFR_PageIndex(S1CON_Page);\
        __STATE__=RI1;\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 clear interrupt flag
* @details      Clear TI1
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_ClearTI1();
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_ClearTI1()\
    MWT(\
        __DRV_SFR_PageIndex(S1CON_Page);\
        TI1=0;\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 clear interrupt flag
* @details      Clear RI1
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_ClearRI1();
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_ClearRI1()\
    MWT(\
        __DRV_SFR_PageIndex(S1CON_Page);\
        RI1=0;\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 transfer data
* @details      Set TB81
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_SetTXData9th();
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_SetTXData9th()\
    MWT(\
        __DRV_SFR_PageIndex(S1CON_Page);\
        TB81=1;\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 transfer data
* @details      Clear TB81
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_ClearTXData9th();
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_ClearTXData9th()\
    MWT(\
        __DRV_SFR_PageIndex(S1CON_Page);\
        TB81=0;\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 read receive data
* @details      Read RB81
* @param[in]    __RXData__ :
* @arg\b        Read 9th data
* @return       None
* @note         Data type bit

* @par          Example
* @code
                __DRV_URT1_GetRXData9th(usetmp);
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_GetRXData9th(__RXData__)\
    MWT(\
        __DRV_SFR_PageIndex(S1CON_Page);\
        __RXData__=RB81;\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 transfer data
* @details      Send S1BUF
* @param[in]    __TXData__:
*  @arg\b       userdata(0~255)
* @return       None
* @note         None
* @par          Example
* @code
                __DRV_URT1_SendTXData(0x55);
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_SendTXData(__TXData__)\
    MWT(\
        __DRV_SFR_PageIndex(S1CON_Page);\
        S1BUF=__TXData__;\
        __DRV_SFR_PageIndex(0);\
    ;)

/**
*******************************************************************************
* @brief        UART1 read receive data
* @details      Read S1BUF
* @param[in]    __RXData__ :
* @arg\b        Read data
* @return       S1BUF
* @note         None
* @par          Example
* @code
                __DRV_URT1_ReceiveRXData(usertmp);
* @endcode
*******************************************************************************
*/

#define __DRV_URT1_ReceiveRXData(__RXData__)\
    MWT(\
        __DRV_SFR_PageIndex(S1CON_Page);\
        __RXData__=S1BUF;\
        __DRV_SFR_PageIndex(0);\
    ;)

///@cond
bool DRV_URT1_GetTI1(void);
bool DRV_URT1_GetRI1(void);
void DRV_URT1_ClearTI1(void);
void DRV_URT1_ClearRI1(void);
void DRV_URT1_SetTXData9th(void);
void DRV_URT1_ClearTXData9th(void);
bool DRV_URT1_GetRXData9th(void);
void DRV_URT1_SendTXData(uint8_t TXData);
uint8_t DRV_URT1_ReceiveRXData(void);
///@endcond


#endif


