/**
 ******************************************************************************
 *
 * @file        MG82F6P32_INT_VECTOR.c
 *
 * @brief       This is the C code format main file.
 *
 * @par         Project
 *              MG82F6P32
 * @version     V1.00
 * @date        2024/03/26
 * @copyright   Copyright (c) 2019 MegaWin Technology Co., Ltd.
 *              All rights reserved.
 *
 ******************************************************************************
 * @par         Disclaimer
 *      The Demo software is provided "AS IS" without any warranty, either
 *      expressed or implied, including, but not limited to, the implied warranties
 *      of merchantability and fitness for a particular purpose. The author will
 *      not be liable for any special, incidental, consequential or indirect
 *      damages due to loss of data or any other reason.
 *      These statements agree with the world wide and local dictated laws about
 *      authorship and violence against these laws.
 ******************************************************************************
 ******************************************************************************
 */

#include "MG82F6P32_CONFIG.h"

/**
 *******************************************************************************
 * Interrupts ISR
 *******************************************************************************
 */

/// @cond __DRV_Wizard_Without_Doxygen
#if MG82F6P32_INT_ALL_WIZARD
/// @endcond


/**
 *******************************************************************************
 * @brief       External Interrupt 0 Vector Service Routine
 * @details
 * @return      None
 * @note
 * @par         Example
 * @code
 *    ExINT0_IE0_ISR();
 * @endcode
 *******************************************************************************
 */
/// @cond __DRV_Wizard_Without_Doxygen
#if MG82F6P32_INT0_VECTOR_CONFIG
/// @endcond
void ExINT0_IE0_ISR(void)  interrupt INT0_ISR_VECTOR
{
    _push_(SFRPI);

    //By User define...

    __DRV_INT0_ClearFlag();

    _pop_(SFRPI);
}
/// @cond __DRV_Wizard_Without_Doxygen
#endif
/// @endcond


/**
 *******************************************************************************
 * @brief       Timer 0 Interrupt Vector Service Routine
 * @details
 * @return      None
 * @note
 * @par         Example
 * @code
 *    TIMER0_TF0_ISR();
 * @endcode
 *******************************************************************************
 */
/// @cond __DRV_Wizard_Without_Doxygen
#if MG82F6P32_TIMER0_IT_VECTOR_CONFIG
/// @endcond
void TIMER0_TF0_ISR(void)   interrupt TIMER0_ISR_VECTOR
{
    _push_(SFRPI);

    //By User define...

    DRV_TIMER0_ClearTF0();

    _pop_(SFRPI);
}
/// @cond __DRV_Wizard_Without_Doxygen
#endif
/// @endcond



/**
 *******************************************************************************
 * @brief       External Interrupt 1 Vector Service Routine
 * @details
 * @return      None
 * @note
 * @par         Example
 * @code
 *    ExINT1_IE1_ISR();
 * @endcode
 *******************************************************************************
 */
/// @cond __DRV_Wizard_Without_Doxygen
#if MG82F6P32_INT1_VECTOR_CONFIG
/// @endcond
void ExINT1_IE1_ISR(void)   interrupt INT1_ISR_VECTOR
{
    _push_(SFRPI);

    //By User define...

    __DRV_INT1_ClearFlag();

    _pop_(SFRPI);
}
/// @cond __DRV_Wizard_Without_Doxygen
#endif
/// @endcond



/**
 *******************************************************************************
 * @brief       Timer 1 Interrupt Vector Service Routine
 * @details
 * @return      None
 * @note
 * @par         Example
 * @code
 *    TIMER1_TF1_ISR();
 * @endcode
 *******************************************************************************
 */
/// @cond __DRV_Wizard_Without_Doxygen
#if MG82F6P32_TIMER1_IT_VECTOR_CONFIG
/// @endcond
void TIMER1_TF1_ISR(void)   interrupt TIMER1_ISR_VECTOR
{
    _push_(SFRPI);

    //By User define...

    DRV_TIMER1_ClearTF1();

    _pop_(SFRPI);
}
/// @cond __DRV_Wizard_Without_Doxygen
#endif
/// @endcond



/**
 *******************************************************************************
 * @brief       UART0 Interrupt Vector Service Routine
 * @details
 * @return      None
 * @note
 * @par         Example
 * @code
 *    UART0_RI0_TI0_ISR();
 * @endcode
 *******************************************************************************
 */
/// @cond __DRV_Wizard_Without_Doxygen
#if MG82F6P32_S0_IT_VECTOR_CONFIG
/// @endcond
void UART0_RI0_TI0_ISR(void)   interrupt S0_ISR_VECTOR
{
    _push_(SFRPI);

    //TX Interrupt
    if (DRV_URT0_GetTI0())
    {
        //By User define...

        DRV_URT0_ClearTI0();
    }

    //RX Interrupt
    if (DRV_URT0_GetRI0())
    {

        //By User define...

        DRV_URT0_ClearRI0();
    }

    _pop_(SFRPI);
}
/// @cond __DRV_Wizard_Without_Doxygen
#endif
/// @endcond



/**
 *******************************************************************************
 * @brief       Timer 2 Interrupt Vector Service Routine
 * @details
 * @return      None
 * @note
 * @par         Example
 * @code
 *    TIMER2_TF2_EXF2_TF2L_ISR();
 * @endcode
 *******************************************************************************
 */
/// @cond __DRV_Wizard_Without_Doxygen
#if MG82F6P32_TIMER2_IT_VECTOR_CONFIG
/// @endcond
void TIMER2_TF2_EXF2_TF2L_ISR(void)   interrupt TIMER2_ISR_VECTOR
{
    _push_(SFRPI);

    if(DRV_TIMER2_GetTF2())
    {
        //By User define...

        DRV_TIMER2_ClearTF2();
    }

    if(DRV_TIMER2_GetEXF2())
    {
        //By User define...

        DRV_TIMER2_ClearEXF2();
    }

    if(DRV_TIMER2_GetTF2L())
    {
        //By User define...

        DRV_TIMER2_ClearTF2L();
    }

    _pop_(SFRPI);
}
/// @cond __DRV_Wizard_Without_Doxygen
#endif
/// @endcond



/**
 *******************************************************************************
 * @brief       External Interrupt 2 Vector Service Routine
 * @details
 * @return      None
 * @note
 * @par         Example
 * @code
 *    ExINT2_IE2_ISR();
 * @endcode
 *******************************************************************************
 */
/// @cond __DRV_Wizard_Without_Doxygen
#if MG82F6P32_INT2_VECTOR_CONFIG
/// @endcond
void ExINT2_IE2_ISR(void)   interrupt INT2_ISR_VECTOR
{
    _push_(SFRPI);

    //By User define...

    __DRV_INT2_ClearFlag();

    _pop_(SFRPI);
}
/// @cond __DRV_Wizard_Without_Doxygen
#endif
/// @endcond



/**
 *******************************************************************************
 * @brief       SPI Interrupt Vector Service Routine
 * @details
 * @return      None
 * @note
 * @par         Example
 * @code
 *    SPI_SPIF_ISR();
 * @endcode
 *******************************************************************************
 */
/// @cond __DRV_Wizard_Without_Doxygen
#if MG82F6P32_SPI_IT_VECTOR_CONFIG
/// @endcond
void SPI_SPIF_ISR(void)   interrupt SPI_ISR_VECTOR
{
    _push_(SFRPI);

    //By User define...

    DRV_SPI_ClearSPIF();

    _pop_(SFRPI);
}
/// @cond __DRV_Wizard_Without_Doxygen
#endif
/// @endcond



/**
 *******************************************************************************
 * @brief       ADC Interrupt Vector Service Routine
 * @details
 * @return      None
 * @note
 * @par         Example
 * @code
 *    ADC_ADCI_ISR();
 * @endcode
 *******************************************************************************
 */
/// @cond __DRV_Wizard_Without_Doxygen
#if MG82F6P32_ADC_IT_VECTOR_CONFIG
/// @endcond
void ADC_ADCI_ADCWI_SMPF_ISR(void)   interrupt ADC_ISR_VECTOR
{
    _push_(SFRPI);

    //ADCI Interrupt
    if(__DRV_ADC_GetFlag())
    {
        //By User define...

        __DRV_ADC_ClearFlag();
    }

    //Window Detect Interrupt
    if(__DRV_ADC_Window_GetFlag())
    {
        //By User define...

        __DRV_ADC_ClearWindowFlag();
    }

    //Sample Flag Interrupt
    if(__DRV_ADC_Sample_GetFlag())
    {
        //By User define...

        __DRV_ADC_ClearSampleFlag();
    }

    _pop_(SFRPI);
}
/// @cond __DRV_Wizard_Without_Doxygen
#endif
/// @endcond



/**
 *******************************************************************************
 * @brief       PCA 0 Interrupt Vector Service Routine
 * @details
 * @return      None
 * @note
 * @par         Example
 * @code
 *    PCA0_CF_CCFn_ISR();
 * @endcode
 *******************************************************************************
 */
/// @cond __DRV_Wizard_Without_Doxygen
#if MG82F6P32_PCA1_IT_VECTOR_CONFIG
/// @endcond
void PCA1_CF_CCFn_ISR(void)   interrupt PCA1_ISR_VECTOR
{
    _push_(SFRPI);

    if(DRV_PCA1_GetOverflowFlag())
    {
        //By User define...

        __DRV_PCA1_Overflow_ClearFlag();
    }

    if (DRV_PCA1_GetModule0Flag())
    {
        //By User define...

        __DRV_PCA1_Module0_ClearFlag();
    }

    if (DRV_PCA1_GetModule1Flag())
    {
        //By User define...

        __DRV_PCA1_Module1_ClearFlag();
    }

    _pop_(SFRPI);
}
/// @cond __DRV_Wizard_Without_Doxygen
#endif
/// @endcond



/**
 *******************************************************************************
 * @brief       System Flag Interrupt Vector Service Routine
 * @details
 * @return      None
 * @note
 * @par         Example
 * @code
 *    SYSFlag_ISR();
 * @endcode
 *******************************************************************************
 */
/// @cond __DRV_Wizard_Without_Doxygen
#if MG82F6P32_SYSFLAG_IT_VECTOR_CONFIG
/// @endcond
void SYSFlag_ISR()  interrupt SYSFLAG_ISR_VECTOR
{
    _push_(SFRPI);

    //WDT Interrupt
    if (__DRV_WDT_GetFlag())
    {
        //By User define...

        __DRV_WDT_ClearFlag();
    }

    //RTC Interrupt
    if (__DRV_RTC_GetFlag())
    {
        //By User define...

        __DRV_RTC_ClearFlag();
    }

    //BOD0 Interrupt
    if (__DRV_BODx_BOD0_GetFlag())
    {
        //By User define...

        __DRV_BODx_BOD0_ClearFlag();
    }

    //BOD1 Interrupt
    if (__DRV_BODx_BOD1_GetFlag())
    {
        //By User define...

        __DRV_BODx_BOD1_ClearFlag();
    }

    //SI2C Interrupt
    if (__DRV_SI2C_GetStartFlag())
    {
        //By User define...

        __DRV_SI2C_ClearStartFlag();
    }

    if (__DRV_SI2C_GetStopFlag())
    {
        //By User define...

        __DRV_SI2C_ClearStopFlag();
    }

    _pop_(SFRPI);
}
/// @cond __DRV_Wizard_Without_Doxygen
#endif
/// @endcond



/**
 *******************************************************************************
 * @brief       KBI Interrupt Vector Service Routine
 * @details
 * @return      None
 * @note
 * @par         Example
 * @code
 *    KBI_KBIF_ISR();
 * @endcode
 *******************************************************************************
 */
/// @cond __DRV_Wizard_Without_Doxygen
#if MG82F6P32_KBI_IT_VECTOR_CONFIG
/// @endcond
void KBI_KBIF_ISR()  interrupt KBI_ISR_VECTOR
{
    _push_(SFRPI);

    //By User define...

    __DRV_KBI_ClearFlag();

    _pop_(SFRPI);
}
/// @cond __DRV_Wizard_Without_Doxygen
#endif
/// @endcond



/**
 *******************************************************************************
 * @brief       I2C0 Interrupt Vector Service Routine
 * @details
 * @return      None
 * @note
 * @par         Example
 * @code
 *    I2C0_SI_ISR();
 * @endcode
 *******************************************************************************
 */
/// @cond __DRV_Wizard_Without_Doxygen
#if MG82F6P32_I2C0_IT_VECTOR_CONFIG
/// @endcond
void I2C0_SI_ISR(void)   interrupt I2C0_ISR_VECTOR
{
    _push_(SFRPI);

    Sample_I2C0_ISR_Handle();

    _pop_(SFRPI);
}
/// @cond __DRV_Wizard_Without_Doxygen
#endif
/// @endcond


/**
 *******************************************************************************
 * @brief       AC0 Interrupt Vector Service Routine
 * @details
 * @return      None
 * @note
 * @par         Example
 * @code
 *    AC0_AC0F_ISR();
 * @endcode
 *******************************************************************************
 */
/// @cond __DRV_Wizard_Without_Doxygen
#if MG82F6P32_AC0_IT_VECTOR_CONFIG
/// @endcond
void AC0_AC0F_ISR(void)   interrupt AC0_ISR_VECTOR
{
    _push_(SFRPI);

    //By User define...

    __DRV_AC0_ClearFlag();

    _pop_(SFRPI);
}
/// @cond __DRV_Wizard_Without_Doxygen
#endif
/// @endcond


/**
 *******************************************************************************
 * @brief       UART1 Interrupt Vector Service Routine
 * @details
 * @return      None
 * @note
 * @par         Example
 * @code
 *    UART1_RI1_STI1_ISR();
 * @endcode
 *******************************************************************************
 */
/// @cond __DRV_Wizard_Without_Doxygen
#if MG82F6P32_S1_IT_VECTOR_CONFIG
/// @endcond
void UART1_RI1_STI1_ISR(void)   interrupt S1_ISR_VECTOR
{
    _push_(SFRPI);

    //TX Interrupt
    if (DRV_URT1_GetTI1())
    {
        //By User define...

        DRV_URT1_ClearTI1();
    }

    //RX Interrupt
    if (DRV_URT1_GetRI1())
    {
        //By User define...

        DRV_URT1_ClearRI1();
    }

    _pop_(SFRPI);
}
/// @cond __DRV_Wizard_Without_Doxygen
#endif
/// @endcond


/**
 *******************************************************************************
 * @brief       External Interrupt 3 Vector Service Routine
 * @details
 * @return      None
 * @note
 * @par         Example
 * @code
 *    ExINT3_IE3_ISR();
 * @endcode
 *******************************************************************************
 */
/// @cond __DRV_Wizard_Without_Doxygen
#if MG82F6P32_INT3_VECTOR_CONFIG
/// @endcond
void ExINT3_IE3_ISR(void)   interrupt INT3_ISR_VECTOR
{
    _push_(SFRPI);

    //By User define...

    __DRV_INT3_ClearFlag();

    _pop_(SFRPI);
}
/// @cond __DRV_Wizard_Without_Doxygen
#endif
/// @endcond


/**
 *******************************************************************************
 * @brief       Timer 3 Interrupt Vector Service Routine
 * @details
 * @return      None
 * @note
 * @par         Example
 * @code
 *    TIMER3_TF3_EXF3_TF3L_ISR();
 * @endcode
 *******************************************************************************
 */
/// @cond __DRV_Wizard_Without_Doxygen
#if MG82F6P32_TIMER3_IT_VECTOR_CONFIG
/// @endcond
void TIMER3_TF3_EXF3_TF3L_ISR(void)   interrupt TIMER3_ISR_VECTOR
{
    _push_(SFRPI);

    if(DRV_TIMER3_GetTF3())
    {
        //By User define...

        DRV_TIMER3_ClearTF3();
    }

    if(DRV_TIMER3_GetEXF3())
    {
        //By User define...

        DRV_TIMER3_ClearEXF3();
    }

    if(DRV_TIMER3_GetTF3L())
    {
        //By User define...

        DRV_TIMER3_ClearTF3L();
    }

    _pop_(SFRPI);
}
/// @cond __DRV_Wizard_Without_Doxygen
#endif
/// @endcond


/**
 *******************************************************************************
 * @brief       AC1 Interrupt Vector Service Routine
 * @details
 * @return      None
 * @note
 * @par         Example
 * @code
 *    AC1_AC1F_ISR();
 * @endcode
 *******************************************************************************
 */
/// @cond __DRV_Wizard_Without_Doxygen
#if MG82F6P32_AC1_OPAx_IT_VECTOR_CONFIG
/// @endcond
void AC1_AC1F_OPA0_OP0CMPF_OPA1_OP1CMPF_ISR(void)   interrupt AC1_OPA0_OPA1_ISR_VECTOR
{
    _push_(SFRPI);

    if(DRV_AC1_GetFlag())
    {
        //By User define...

        __DRV_AC1_ClearFlag();
    }

    if(DRV_OPA0_GetFlag())
    {
        //By User define...

        __DRV_OPA0_ClearFlag();
    }

    if(DRV_OPA1_GetFlag())
    {
        //By User define...

        __DRV_OPA1_ClearFlag();
    }

    _pop_(SFRPI);
}
/// @cond __DRV_Wizard_Without_Doxygen
#endif
/// @endcond



/**
 *******************************************************************************
 * @brief       DMA Interrupt Vector Service Routine
 * @details
 * @return      None
 * @note
 * @par         Example
 * @code
 *    DMA_DCF0_ISR();
 * @endcode
 *******************************************************************************
 */
/// @cond __DRV_Wizard_Without_Doxygen
#if MG82F6P32_DMA_IT_VECTOR_CONFIG
/// @endcond
void DMA_DCF0_ISR(void)   interrupt DMA_ISR_VECTOR
{
    _push_(SFRPI);

    if (DRV_DMA_GetFlag())
    {
        //By User define...

        __DRV_DMA_ClearCompleteFlag();
    }

    //TF5 Interrupt
    if (DRV_TIMER5_GetTF5())
    {
        //By User define...

        DRV_TIMER5_ClearTF5();
    }

    //TF6 Interrupt
    if (DRV_TIMER6_GetTF6())
    {
        //By User define...

        DRV_TIMER6_ClearTF6();
    }

    _pop_(SFRPI);
}
/// @cond __DRV_Wizard_Without_Doxygen
#endif
/// @endcond



/**
 *******************************************************************************
 * @brief       PTM0 Interrupt Vector Service Routine
 * @details
 * @return      None
 * @note
 * @par         Example
 * @code
 *    PTM0_CF_CCFx_ISR();
 * @endcode
 *******************************************************************************
 */
/// @cond __DRV_Wizard_Without_Doxygen
#if MG82F6P32_PTM0_IT_VECTOR_CONFIG
/// @endcond
void PTM0_CF_CCFx_ISR(void)   interrupt PTM0_ISR_VECTOR
{
    _push_(SFRPI);

    if (DRV_PTM0_GetOverflowFlag())
    {
        //By User define...

        __DRV_PTM0_Overflow_ClearFlag();
    }

    if (DRV_PTM0_GetModule0Flag())
    {
        //By User define...

        __DRV_PTM0_Module0_ClearFlag();
    }

    if (DRV_PTM0_GetModule1Flag())
    {
        //By User define...

        __DRV_PTM0_Module1_ClearFlag();
    }

    if (DRV_PTM0_GetModule2Flag())
    {
        //By User define...

        __DRV_PTM0_Module2_ClearFlag();
    }

    if (DRV_PTM0_GetModule3Flag())
    {
        //By User define...

        __DRV_PTM0_Module3_ClearFlag();
    }

    if (DRV_PTM0_GetModule4Flag())
    {
        //By User define...

        __DRV_PTM0_Module4_ClearFlag();
    }

    if (DRV_PTM0_GetModule5Flag())
    {
        //By User define...

        __DRV_PTM0_Module5_ClearFlag();
    }

    _pop_(SFRPI);
}
/// @cond __DRV_Wizard_Without_Doxygen
#endif
/// @endcond


/**
 *******************************************************************************
 * @brief       I2C1 Interrupt Vector Service Routine
 * @details
 * @return      None
 * @note
 * @par         Example
 * @code
 *    I2C1_SI1_ISR();
 * @endcode
 *******************************************************************************
 */
/// @cond __DRV_Wizard_Without_Doxygen
#if MG82F6P32_I2C1_IT_VECTOR_CONFIG
/// @endcond
void I2C1_SI1_ISR(void)   interrupt I2C1_ISR_VECTOR
{
    _push_(SFRPI);

    Sample_I2C1_ISR_Handle();

    _pop_(SFRPI);
}
/// @cond __DRV_Wizard_Without_Doxygen
#endif
/// @endcond


/// @cond __DRV_Wizard_Without_Doxygen
#endif
/// @endcond


